<?php

namespace App\Controllers;

use App\baseClasses\KCBase;
use App\baseClasses\KCRequest;
use App\models\KCService;
use App\models\KCServiceDoctorMapping;
use Automattic\WooCommerce\Blocks\RestApi;
use App\models\KCReceptionistClinicMapping;
use App\models\KCClinic;
use Exception;
use WP_User;
class KCServiceController extends KCBase {

	public $db;
	/**
	 * @var KCRequest
	 */
	private $request;

	public function __construct() {

		global $wpdb;

		$this->db = $wpdb;

		$this->request = new KCRequest();

	}

	public function index() {

		$request_data      = $this->request->getInputs();
        $search_condition         = '';
		$service_table     = $this->db->prefix . 'kc_services';
		$service_doctor_mapping  = $this->db->prefix . 'kc_service_doctor_mapping' ;
		$users_table       = $this->db->prefix . 'users';
		$clinic_doctor_mapping = $this->db->prefix.'kc_doctor_clinic_mappings';
        $login_user = wp_get_current_user();

		$service_count_query = "SELECT count(*) AS count FROM {$service_table}";

		$total_services = $this->db->get_var( $service_count_query, OBJECT );

		if ( !empty($request_data['searchKey']) && !empty($request_data['searchValue'])) {
			$search_condition = " AND {$service_table}.{$request_data['searchKey']}  LIKE  '%{$request_data['searchValue']}%' ";
		}

		$doctor_condition = "" ;
        $zoom_config_data = "" ;
		$clinic_condition = "";
		$clinic_join = "";
        $doctor_googlemeet= '';

		if(isset($login_user->roles[0]) && $this->getDoctorRole() === $login_user->roles[0]) {

			$doctor_condition = " AND {$service_doctor_mapping}.doctor_id = {$login_user->data->ID} " ;

			$zoom_config_data = get_user_meta($login_user->data->ID, 'zoom_config_data', true);

        	$zoom_config_data = json_decode($zoom_config_data);

            $doctor_googlemeet = get_user_meta($login_user->data->ID, KIVI_CARE_PREFIX.'google_meet_connect',true);
            $doctor_googlemeet = $doctor_googlemeet == 'off' || empty($doctor_googlemeet) ? 'off' : 'on';

        } else if(isset($request_data['doctor_id'])) {
            $request_data['doctor_id'] = (int)$request_data['doctor_id'];
			$doctor_condition = " AND {$service_doctor_mapping}.doctor_id = " . $request_data['doctor_id'] ;

            $zoom_config_data = get_user_meta($request_data['doctor_id'], 'zoom_config_data', true);

            $zoom_config_data = json_decode($zoom_config_data);

            $doctor_googlemeet = get_user_meta($request_data['doctor_id'], KIVI_CARE_PREFIX.'google_meet_connect',true);
            $doctor_googlemeet = $doctor_googlemeet == 'off' || empty($doctor_googlemeet) ? 'off' : 'on';

        }elseif(isset($login_user->roles[0]) && $this->getClinicAdminRole() === $login_user->roles[0]){
            $clinic_id = 0;
            if(isKiviCareProActive()){
				$clinic_join = " JOIN  {$clinic_doctor_mapping} ON {$clinic_doctor_mapping}.doctor_id = {$service_doctor_mapping}.doctor_id";
				$clinic =  (new KCClinic())->get_by([ 'clinic_admin_id' => get_current_user_id()]);
				if(isset($clinic[0]->id)) {
					$clinic_id = (int)$clinic[0]->id ;
				}
            }
            else{
                $clinic_id = kcGetDefaultClinicId();
            }
            $clinic_condition = " AND {$clinic_doctor_mapping}.clinic_id = " . $clinic_id ;
        }elseif(isset($login_user->roles[0]) && $this->getReceptionistRole() === $login_user->roles[0]){
            $clinic_id = 0;
			if(isKiviCareProActive()){
				$clinic_join = " JOIN  {$clinic_doctor_mapping} ON {$clinic_doctor_mapping}.doctor_id = {$service_doctor_mapping}.doctor_id";
				$clinic =  (new KCReceptionistClinicMapping())->get_by([ 'receptionist_id' => get_current_user_id()]);
				if(isset($clinic[0]->clinic_id)) {
					$clinic_id = (int)$clinic[0]->clinic_id ;
				}
            }
            else{
                $clinic_id = kcGetDefaultClinicId();
            }
            $clinic_condition = " AND {$clinic_doctor_mapping}.clinic_id = " . $clinic_id ;
        }
	
		$active_services = '' ;

		// get active services condition.
		if(isset($request_data['module_type']) && $request_data['module_type'] === 'appointment_form') {
			$active_services = " AND {$service_table}.status = '1' ";
		}
	
		if(($this->isTeleMedActive() && isset($zoom_config_data->enableTeleMed) && strval($zoom_config_data->enableTeleMed) == 'true') || (isKiviCareGoogleMeetActive() && $doctor_googlemeet == 'on')) {
			$query = "SELECT {$service_doctor_mapping}.*, {$service_table}.name AS name, {$service_table}.type AS service_type, {$service_table}.created_at AS created_at,  {$users_table}.display_name AS doctor_name  FROM {$service_doctor_mapping}
					JOIN {$service_table}
					ON {$service_doctor_mapping}.service_id = {$service_table}.id
					JOIN {$users_table}
					ON {$users_table}.ID = {$service_doctor_mapping}.doctor_id
					{$clinic_join}
					WHERE 0 = 0  {$doctor_condition}  {$clinic_condition} {$active_services} {$search_condition} ORDER BY {$service_table}.id  DESC" ;
		}else{
			$query = "  SELECT {$service_doctor_mapping}.*, {$service_table}.name AS name, {$service_table}.type AS service_type,{$service_table}.created_at AS created_at, {$users_table}.display_name AS doctor_name  FROM {$service_doctor_mapping}
			JOIN {$service_table}
			ON {$service_doctor_mapping}.service_id = {$service_table}.id
			JOIN {$users_table}
			ON {$users_table}.ID = {$service_doctor_mapping}.doctor_id
			{$clinic_join}
			WHERE 0 = 0 {$doctor_condition} {$clinic_condition} {$active_services} AND {$service_table}.type != 'system_service' {$search_condition} ORDER BY {$service_table}.id  DESC" ;
		}


		$services = collect($this->db->get_results( $query, OBJECT ));

		$services = $services->map( function ( $services )  {
			$services->service_type = isset( $services->service_type ) ? str_replace( '_', ' ', $services->service_type) : "";
			return $services;
		} );

		if (!empty($request_data['searchKey']) && !empty($request_data['searchValue'])) {
			$total_rows = count( $services );
		} else {
			$total_rows = $total_services;
		}

		if ( ! count( $services ) ) {
			echo json_encode( [
				'status'  => false,
				'message' => esc_html__('No services found', 'kc-lang'),
				'data'    => []
			] );
			wp_die();
		}

		echo json_encode( [
			'status'     => true,
			'message'    => esc_html__('Service list', 'kc-lang'),
			'data'       => $services,
			'total_rows' => $total_rows
		] );

	}

	public function save() {

		global $wpdb;

		if ( ! kcCheckPermission( 'service_add' ) ) {
			echo json_encode( [
				'status'      => false,
				'status_code' => 403,
				'message'     => esc_html__('You don\'t have a permission to access', 'kc-lang'),
				'data'        => []
			] );
			wp_die();
		}

		$request_data = $this->request->getInputs();

		$temp = [
			'name'   => $request_data['name'],
			'price'  => (int)$request_data['price'],
			'type'   => $request_data['type']['id'],
			'status' => $request_data['status']['id'],
		];
		$static_data_table =  $wpdb->prefix . 'kc_static_data';
        $request_data['type']['id'] = (int)$request_data['type']['id'];
		$query = "SELECT * FROM {$static_data_table} WHERE id = '{$request_data['type']['id']}' " ;
		$result = $wpdb->get_row($query);

		if(!empty($result)) {
			$temp['type'] = $result->value ;
		}

		$service = new KCService();

		$service_doctor_mapping = new KCServiceDoctorMapping();

		if (!isset( $request_data['id'])) {

			$temp['created_at'] = current_time( 'Y-m-d H:i:s' );
			$service_id = $service->insert( $temp );
		
            if ($service_id) {
				$user_id = get_current_user_id();
				$user_obj = new WP_User($user_id);
				if($user_obj->roles[0] == 'kiviCare_doctor') {
					$service_mapping_data = [
                        'service_id' => (int)$service_id,
                        'clinic_id'  => kcGetDefaultClinicId(),
						'doctor_id'  => (int)$user_id,
						'charges'    => (int)$request_data['price'],
						'status'     => (int)$request_data['status']['id'],
                    ];

					$service_doctor_mapping->insert($service_mapping_data);
					// hook for service add.
					do_action( 'kc_service_add', $service_mapping_data );
				} else {
					foreach ($request_data['doctor_id'] as $key => $val) {
						$service_mapping_data = [
							'service_id' => (int)$service_id,
							'clinic_id'  => kcGetDefaultClinicId(),
							'doctor_id'  => (int)$val['id'],
							'charges'    => (int)$request_data['price'],
							'status'     => (int)$request_data['status']['id'],
                    	] ;
                        $service_doctor_mapping->insert($service_mapping_data);
						// hook for service add.
						do_action( 'kc_service_add', $service_mapping_data );
                	}
				}
            }

			$message            = esc_html__('Service has been saved successfully', 'kc-lang');

		} else {

			$user = wp_get_current_user();
            $request_data['service_id'] = (int)$request_data['service_id'];
            $service->update([
				'name' => $request_data['name']
			], array('id' => $request_data['service_id']));
			
			$appointments_service_table = $this->db->prefix . 'kc_service_doctor_mapping';
            $service_data = $this->db->get_results('select id from '.$appointments_service_table.' where service_id='.$request_data['service_id'] );
			if($service_data != null && count($service_data) >0 ){
				foreach ($service_data as $s){
					$product_id = $this->getProductIdOfService($s->id);
					if($product_id != null &&  get_post_status( $product_id )){
						$my_post = array(
							'ID'           => $product_id,
							'post_title'   => $request_data['name'],
							);
						wp_update_post( $my_post );
					}
				}
			}

			$service_mapping_update_data  = [
				'service_id' => $request_data['service_id'],
				'doctor_id' => (int)$request_data['doctor_id']['id'],
				'charges'    => (int)$request_data['price'],
                'status'    => (int)$request_data['status']['id'],
			];

            $request_data['id'] = (int)$request_data['id'];
            $service_doctor_mapping->update($service_mapping_update_data, array('id' => $request_data['id']));

			$service_mapping_update_data  = [
				'id' => $request_data['id'],
				'service_id' => $request_data['service_id'],
				'doctor_id' => (int)$request_data['doctor_id']['id'],
				'charges'    => (int)$request_data['price'],
                'status'    => (int)$request_data['status']['id'],
			];

			do_action( 'kc_service_update', $service_mapping_update_data);

            $product_id = $this->getProductIdOfService($request_data['id']);
            if($product_id != null &&  get_post_status( $product_id )){
                update_post_meta($product_id,'_price', $request_data['price']);
                update_post_meta($product_id,'_sale_price', $request_data['price']);
            }
            $service->update( $temp, array( 'id' => $request_data['id'] ) );
            $message = esc_html__('Service has been updated successfully', 'kc-lang');

		}

		echo json_encode( [
			'status'  => true,
			'message' => $message
		] );

	}

	public function edit() {

		
		if ( ! kcCheckPermission( 'service_edit' ) || ! kcCheckPermission( 'service_view' ) ) {
			
			echo json_encode( [
				'status'      => false,
				'status_code' => 403,
				'message'     => esc_html__('You don\'t have a permission to access', 'kc-lang'),
				'data'        => []
			] );
			
			wp_die();
		}

		$request_data = $this->request->getInputs();

		try {

			if ( ! isset( $request_data['id'] ) ) {
				throw new Exception( esc_html__('Data not found', 'kc-lang'), 400 );
			}

			$edit_id = (int)$request_data['id'];
			$service_table     = $this->db->prefix . 'kc_services';
			$static_data_table = $this->db->prefix . 'kc_static_data';
			$service_doctor_mapping  = $this->db->prefix . 'kc_service_doctor_mapping' ;
			$users_table       = $this->db->prefix . 'users';
			
			$query = " SELECT {$service_doctor_mapping}.id AS mapping_id, {$service_table}.id AS service_id, 
                              {$service_doctor_mapping}.*, {$service_table}.* , {$users_table}.display_name AS doctor_name, 
                              {$service_doctor_mapping}.charges AS doctor_charges, 
                              {$service_doctor_mapping}.status AS mapping_status
                       FROM  {$service_doctor_mapping} 
					   JOIN  {$users_table} ON {$users_table}.ID = {$service_doctor_mapping}.doctor_id  
					   JOIN  {$service_table} ON {$service_table}.id = {$service_doctor_mapping}.service_id  
					   WHERE {$service_doctor_mapping}.id = {$edit_id} ";


			$service = $this->db->get_results( $query, OBJECT );

			if ( count( $service ) ) {

				$service = $service[0];

				$service_category_query = "SELECT * FROM  {$static_data_table} WHERE value = '{$service->type}' LIMIT 1 " ;

                $service_category = $this->db->get_results( $service_category_query, OBJECT );

                $status =  new \stdClass();
				$status->id =  0 ;
				$status->label = 'Inactive' ;
				
				if((int) $service->mapping_status === 1) {
					$status->id = 1 ;
					$status->label = 'Active' ;
				}

				$temp = [
					'id'     => $service->mapping_id,
					'service_id' => $service->service_id,
					'name'   => $service->name,
					'price'  => $service->doctor_charges,
                    'doctor_id' =>  [
						'id' 	=>  $service->doctor_id,
						'label' =>  $service->doctor_name
					],
					'type'   => [
						'id'    => $service_category[0]->type,
						'label' => $service_category[0]->label
					],
					'status' => $status,
				];

                echo json_encode( [
					'status'  => true,
					'message' => esc_html__('Service data', 'kc-lang'),
					'data'    => $temp
				] );

			} else {
				throw new Exception( esc_html__('Data not found', 'kc-lang'), 400 );
			}


		} catch ( Exception $e ) {
			$code    = $e->getCode();
			$message = $e->getMessage();
			header( "Status: $code $message" );
			echo json_encode( [
				'status'  => false,
				'message' => $e->getMessage()
			] );
		}
	}

	public function delete() {

		if ( ! kcCheckPermission( 'service_delete' ) ) {
			echo json_encode( [
				'status'      => false,
				'status_code' => 403,
				'message'     => esc_html__('You don\'t have a permission to access', 'kc-lang'),
				'data'        => []
			] );
			wp_die();
		}

		$request_data = $this->request->getInputs();

		try {

			if ( ! isset( $request_data['id'] ) ) {
				throw new Exception( esc_html__('Data not found', 'kc-lang'), 400 );
			}

            $service_doctor_mapping = new KCServiceDoctorMapping();

            $request_data['id'] = (int)$request_data['id'];
            $product_id = $this->getProductIdOfService($request_data['id']);
            if($product_id != null && get_post_status( $product_id )){
				do_action( 'kc_woocoomerce_service_delete', $product_id );
                wp_delete_post($product_id);
            }

			$id = $request_data['id'];
			$results = $service_doctor_mapping->delete( [ 'id' => $id ] );

			if ( $results ) {
				do_action( 'kc_service_delete', $id);
				echo json_encode( [
					'status'  => true,
					'message' => esc_html__('Service has been deleted successfully', 'kc-lang'),
				] );
			} else {
				throw new Exception( esc_html__('Service delete failed', 'kc-lang'), 400 );
			}


		} catch ( Exception $e ) {

			$code    = $e->getCode();
			$message = $e->getMessage();

			header( "Status: $code $message" );

			echo json_encode( [
				'status'  => false,
				'message' => $message
			] );
		}
	}

	public function clinicService () {
	    $table = $this->db->prefix  . 'kc_services' ;
        $query = "SELECT `id`, `type`, `name`, `price` FROM {$table} " ;
        $services = $this->db->get_results( $query, OBJECT );
        echo json_encode([
            'status' => true,
            'message' => esc_html__('Clinic service list', 'kc-lang'),
            'data' => $services
        ]);
    }

	public function getProductIdOfService($id){
        $id = (int)$id;
		$product_id = '';
		$appointments_service_table =  $this->db->prefix . 'kc_service_doctor_mapping';
		$data = $this->db->get_var('select extra from '.$appointments_service_table.' where id='.$id);
        if($data != null){
			$data = json_decode($data);
            $product_id = $data->product_id;
		}	
		return $product_id;
	}
}


